﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Media.MediaHandlers.Facade;
using Ozeki.Common;
using System.Diagnostics;


namespace SoftphoneExample
{
    public partial class Softphone : Form
    {
        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        PhoneLineState phoneLineInformation;
        IPhoneCall call;
        Microphone microphone = Microphone.GetDefaultDevice();
        Speaker speaker = Speaker.GetDefaultDevice();
        MediaConnector connector = new MediaConnector();
        PhoneCallAudioSender mediaSender = new PhoneCallAudioSender();
        PhoneCallAudioReceiver mediaReceiver = new PhoneCallAudioReceiver();
        bool inComingCall;

        
        public Softphone()
        {
            InitializeComponent();
            //some devices are missing
            string message = String.Empty;

            if (microphone == null)
                message += "You have no microphone attached to your computer, please note that your partner will not hear your voice.\n";
            if (speaker == null)
                message += "You have no speaker attached to your computer, please note that you will not hear your partner.\n";

            if (message != String.Empty)
                MessageBox.Show(message, "Some devices are missing", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone line state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineInformation = e.Item;
            InvokeGUIThread(() =>
            {
                label2.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                   label3.Text = "Registration succeeded";
                   label1.Text = "Online";

                }
                else
                    label1.Text = e.Item.ToString();
            });
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
            {
                label1.Text = "Incoming call ";
                textBox1.Text += String.Format("from {0}", e.Item.DialInfo);
                call = e.Item;
                WireUpCallEvents();
                inComingCall = true;
            });
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { label1.Text = e.Item.ToString(); });

            switch (e.Item)
            {
                case CallState.Answered:
                    if (microphone != null)
                        microphone.Start();
                    connector.Connect(microphone, mediaSender);
                    if (speaker != null)
                        speaker.Start();
                    connector.Connect(mediaReceiver, speaker);

                    mediaSender.AttachToCall(call);
                    mediaReceiver.AttachToCall(call);

                    break;
                case CallState.Completed:
                    if (microphone != null)
                        microphone.Stop();
                    connector.Disconnect(microphone, mediaSender);
                    if (speaker != null)
                        speaker.Stop();
                    connector.Disconnect(mediaReceiver, speaker);

                    mediaSender.Detach();
                    mediaReceiver.Detach();

                    WireDownCallEvents();
                    call = null;

                    InvokeGUIThread(() => { textBox1.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    WireDownCallEvents();
                    call = null;
                    break;
            }
        }

        /// <summary>
        /// Displays DTMF signals
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {
            DtmfInfo dtmfInfo = e.Item;

            InvokeGUIThread(() =>
            {
                label1.Text = String.Format("DTMF signal received: {0} ", dtmfInfo.Signal.Signal);
            });
        }

        /// <summary>
        /// There are certain situations when the call cannot be created, for example the dialed number is not available 
        /// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
        /// This event handling is for displaying these events.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
            {
                label1.Text = e.Item.ToString();
            });
        }

        #endregion

        #region Keypad events

        /// <summary>
        /// It starts a call with the dialed number or in case of an incoming call it accepts, picks up the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPickUp_Click(object sender, EventArgs e)
        {
            if (inComingCall)
            {
                inComingCall = false;
                call.Accept();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(textBox1.Text))
                return;

            if (phoneLineInformation != PhoneLineState.RegistrationSucceeded && phoneLineInformation != PhoneLineState.NoRegNeeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }

            call = softPhone.CreateCallObject(phoneLine, textBox1.Text);
            WireUpCallEvents();
            call.Start();
        }

        /// <summary>
        /// In case a call is in progress, it breaks the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                if (inComingCall && call.CallState==CallState.Ringing)
                    call.Reject();
                else
                    call.HangUp();
                inComingCall = false;
                call = null;

            }
            textBox1.Text = string.Empty;
        }

        /// <summary>
        /// It makes the dialing number.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_Click(object sender, EventArgs e)
        {
            var btn = sender as Button;
            if (btn != null)
            {
                if (btn.Text == "0/+")
                {
                    textBox1.Text += "0";
                }
                else
                    textBox1.Text += btn.Text.Trim();
            }
        }

        /// <summary>
        ///  It sends a DTMF signal according to the RFC 2833 standard.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_MouseDown(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StartDTMFSignal((DtmfNamedEvents)id);
                    }
                }
            }
        }
        /// <summary>
        /// It stops sending the given DTMF signal. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPad_MouseUp(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StopDTMFSignal((DtmfNamedEvents)id);
                    }
                }
            }
        }

        #endregion

        #region Helper Functions
        /// <summary>
        ///It initializes a softphone object with a SIP BPX, and it is for requisiting a SIP account that is nedded for a SIP PBX service. It registers this SIP
        ///account to the SIP PBX through an ’IphoneLine’ object which represents the telephoneline. 
        ///If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);
                phoneLine = softPhone.CreatePhoneLine(new SIPAccount(true, "oz876", "oz876", "oz876", "oz876", "192.168.112.100", 5060), new NatConfiguration(NatTraversalMethod.None));
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);

               
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);

                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if (ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
            }
        }

        /// <summary>
        ///  It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents()
        {
            call.CallStateChanged += (call_CallStateChanged);
            call.DtmfReceived += (call_DtmfReceived);
            call.CallErrorOccured += (call_CallErrorOccured);
        }

        /// <summary>
        /// It signs down from the necessary events of a call transact.
        /// </summary>
        private void WireDownCallEvents()
        {
            if (call != null)
            {
                call.CallStateChanged -= (call_CallStateChanged);
                call.DtmfReceived -= (call_DtmfReceived);
                call.CallErrorOccured -= (call_CallErrorOccured);
            }

        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }
        #endregion

        

        private void PhoneMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            softPhone.Close();
        }


        #region Wav Functions
        //playing wav
        private WaveStreamPlayback WaveStreamPlayback;
        private string prevaudiofile;

        private void buttonOpen_Click(object sender, EventArgs e)
        {
            buttonStartPlayback.Text = "Start";
            textBoxRecordingFile.Text = "";
            using (OpenFileDialog ofd = new OpenFileDialog() { Multiselect = false, Filter = "Wave Files (*.wav)|*.wav" })
            {
                if (ofd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    textBoxPlaybackFile.Text = ofd.FileName;
            }

            buttonStopPlayback.PerformClick();
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            textBoxRecordingFile.Text = string.Empty;
            if (!string.IsNullOrEmpty(textBoxPlaybackFile.Text))
            {
                if (WaveStreamPlayback == null || prevaudiofile != textBoxPlaybackFile.Text)
                {
                    try
                    {
                        buttonStopPlayback.PerformClick();
                        WaveStreamPlayback = new WaveStreamPlayback(textBoxPlaybackFile.Text);
                        WaveStreamPlayback.Stopped += (WaveStreamPlayback_Stopped);
                        prevaudiofile = textBoxPlaybackFile.Text;
                        speaker.Start();
                        connector.Connect(WaveStreamPlayback, speaker);
                    }
                    catch (Exception ex)
                    {
                        if (WaveStreamPlayback == null)
                        {
                            MessageBox.Show(String.Format("The file location of your .wav file is incorrect.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                     MessageBoxIcon.Error);
                            return;
                        }else
                            MessageBox.Show(String.Format(ex.Message), string.Empty, MessageBoxButtons.OK,
                     MessageBoxIcon.Error);
                    }
                }
                if (!WaveStreamPlayback.IsStreaming)
                {
                    buttonStartPlayback.Text = "Pause";
                    WaveStreamPlayback.StartStreaming();
                }
                else
                {
                    buttonStartPlayback.Text = "Start";
                    WaveStreamPlayback.PauseStreaming();
                }
            }
        }

        void WaveStreamPlayback_Stopped(object sender, EventArgs e)
        {
            if (PlaybackStopped != null)
                PlaybackStopped(this, EventArgs.Empty);

            InvokeGUIThread(() => buttonStartPlayback.Text = "Start");
        }

        public event EventHandler<EventArgs> PlaybackStopped;


        private void buttonStop_Click(object sender, EventArgs e)
        {
            textBoxRecordingFile.Text = string.Empty;
            if (WaveStreamPlayback != null)
            {
                WaveStreamPlayback.StopStreaming();
                buttonStartPlayback.Text = "Start";
                speaker.Stop();
                connector.Disconnect(WaveStreamPlayback, speaker);
                WaveStreamPlayback.Dispose();
                WaveStreamPlayback = null;
            }
        }

        //recording wav

        private void buttonSave_Click(object sender, EventArgs e)
        {
            textBoxPlaybackFile.Text = string.Empty;
            using (SaveFileDialog sfd = new SaveFileDialog() { Filter = "Wave Files (*.wav)|*.wav", FileName = textBoxRecordingFile.Text })
            {
                if (sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    textBoxRecordingFile.Text = sfd.FileName;
            }
        }

        WaveStreamRecorder WaveStreamRecorder;

        private void buttonStartRecording_Click(object sender, EventArgs e)
        {
            textBoxPlaybackFile.Text = string.Empty;
            if (WaveStreamRecorder == null)
            {
                if (string.IsNullOrEmpty(textBoxRecordingFile.Text))
                    return; // Select a file first

                WaveStreamRecorder = new WaveStreamRecorder(textBoxRecordingFile.Text);
                WaveStreamRecorder.Stopped += (WaveStreamRecorder_Stopped);
                if (microphone != null) ;
                microphone.Start();
                connector.Connect(microphone, WaveStreamRecorder);
            }
            if (!WaveStreamRecorder.IsStreaming)
            {
                buttonStartRecording.Text = "Pause";
                WaveStreamRecorder.StartStreaming();
            }
            else
            {
                buttonStartRecording.Text = "Start";
                WaveStreamRecorder.PauseStreaming();
            }
        }
        
        void WaveStreamRecorder_Stopped(object sender, EventArgs e)
        {
            if (RecorderStopped != null)
                RecorderStopped(this, EventArgs.Empty);

            InvokeGUIThread(() => buttonStartRecording.Text = "Start");
        }

        public event EventHandler<EventArgs> RecorderStopped;

        private void buttonStopRecording_Click(object sender, EventArgs e)
        {
            textBoxPlaybackFile.Text = string.Empty;
            WaveStreamRecorder.StopStreaming();
            connector.Disconnect(microphone, WaveStreamRecorder);
            if (microphone != null)
                microphone.Stop();
            WaveStreamRecorder.Dispose();
            WaveStreamRecorder = null;
            buttonStartRecording.Text = "Start";
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }
        
        #endregion

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void Form1_Load_1(object sender, EventArgs e)
        {
            InitializeSoftPhone();
        }

        private void label4_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/p_272-voip-softphone-development-voip.html");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }






    }

}
